@@ -155,7 +155,7 @@ def build_args(parsed_args, playbooks,
 
 def run_playbooks(parsed_args, playbooks,
                   extra_vars=None, limit=None, tags=None, quiet=False,
-                  verbose_level=None, check=None):
+                  check_output=False, verbose_level=None, check=None):
     """Run a Kayobe Ansible playbook."""
     _validate_args(parsed_args, playbooks)
     cmd = build_args(parsed_args, playbooks,
@@ -168,10 +168,12 @@ def run_playbooks(parsed_args, playbooks,
     # playbooks.
     env.setdefault(CONFIG_PATH_ENV, parsed_args.config_path)
     try:
-        utils.run_command(cmd, quiet=quiet, env=env)
+        utils.run_command(cmd, check_output=check_output, quiet=quiet, env=env)
     except subprocess.CalledProcessError as e:
         LOG.error("Kayobe playbook(s) %s exited %d",
                   ", ".join(playbooks), e.returncode)
+        if check_output:
+            LOG.error("The output was:\n%s", e.output)
         sys.exit(e.returncode)
 
 
@@ -196,7 +198,7 @@ def config_dump(parsed_args, host=None, hosts=None, var_name=None,
         # Don't use check mode for configuration dumps as we won't get any
         # results back.
         run_playbook(parsed_args, "ansible/dump-config.yml",
-                     extra_vars=extra_vars, tags=tags, quiet=True,
+                     extra_vars=extra_vars, tags=tags, check_output=True,
                      verbose_level=verbose_level, check=False)
         hostvars = {}
         for path in os.listdir(dump_dir):
