@@ -19,7 +19,6 @@ from cliff.command import Command
 
 from kayobe import ansible
 from kayobe import kolla_ansible
-from kayobe import utils
 from kayobe import vault
 
 
@@ -114,13 +113,13 @@ class ControlHostBootstrap(KayobeAnsibleMixin, VaultMixin, Command):
     """Bootstrap the Kayobe control environment.
 
     * Downloads and installs Ansible roles from Galaxy.
-    * Generates an SSH key for the ansible control host, if one does not exist.
-    * Installs kolla-ansible on the ansible control host.
+    * Generates an SSH key for the Ansible control host, if one does not exist.
+    * Installs kolla-ansible on the Ansible control host.
     """
 
     def take_action(self, parsed_args):
-        self.app.LOG.debug("Bootstrapping Kayobe control host")
-        utils.galaxy_install("requirements.yml", "ansible/roles")
+        self.app.LOG.debug("Bootstrapping Kayobe Ansible control host")
+        ansible.install_galaxy_roles(parsed_args)
         playbooks = _build_playbook_list("bootstrap")
         self.run_kayobe_playbooks(parsed_args, playbooks)
         playbooks = _build_playbook_list("kolla-ansible")
@@ -131,15 +130,17 @@ class ControlHostUpgrade(KayobeAnsibleMixin, VaultMixin, Command):
     """Upgrade the Kayobe control environment.
 
     * Downloads and installs updated Ansible roles from Galaxy.
-    * Generates an SSH key for the ansible control host, if one does not exist.
-    * Updates kolla-ansible on the ansible control host.
+    * Generates an SSH key for the Ansible control host, if one does not exist.
+    * Updates kolla-ansible on the Ansible control host.
     """
 
     def take_action(self, parsed_args):
-        self.app.LOG.debug("Upgrading Kayobe control host")
+        self.app.LOG.debug("Upgrading Kayobe Ansible control host")
+        # Remove roles that are no longer used. Do this before installing new
+        # ones, just in case a custom role dependency includes any.
+        ansible.prune_galaxy_roles(parsed_args)
         # Use force to upgrade roles.
-        utils.galaxy_install("requirements.yml", "ansible/roles",
-                             force=True)
+        ansible.install_galaxy_roles(parsed_args, force=True)
         playbooks = _build_playbook_list("bootstrap")
         self.run_kayobe_playbooks(parsed_args, playbooks)
         playbooks = _build_playbook_list("kolla-ansible")
@@ -266,6 +267,7 @@ class SeedHypervisorHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
     * Configure a user account for use by kayobe for SSH access.
     * Optionally, create a virtualenv for remote target hosts.
     * Configure user accounts, group associations, and authorised SSH keys.
+    * Configure a PyPI mirror.
     * Configure Yum repos.
     * Configure the host's network interfaces.
     * Set sysctl parameters.
@@ -286,8 +288,8 @@ class SeedHypervisorHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
             sys.exit(1)
         playbooks = _build_playbook_list(
             "ip-allocation", "ssh-known-host", "kayobe-ansible-user",
-            "kayobe-target-venv", "users", "yum", "dev-tools", "network",
-            "sysctl", "ntp", "seed-hypervisor-libvirt-host")
+            "pip", "kayobe-target-venv", "users", "yum", "dev-tools",
+            "network", "sysctl", "ntp", "seed-hypervisor-libvirt-host")
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   limit="seed-hypervisor")
 
@@ -349,6 +351,7 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     * Optionally, create a virtualenv for remote target hosts.
     * Optionally, wipe unmounted disk partitions (--wipe-disks).
     * Configure user accounts, group associations, and authorised SSH keys.
+    * Configure a PyPI mirror.
     * Configure Yum repos.
     * Disable SELinux.
     * Configure the host's network interfaces.
@@ -394,7 +397,7 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         # Run kayobe playbooks.
         playbooks = _build_playbook_list(
             "ip-allocation", "ssh-known-host", "kayobe-ansible-user",
-            "kayobe-target-venv")
+            "pip", "kayobe-target-venv")
         if parsed_args.wipe_disks:
             playbooks += _build_playbook_list("wipe-disks")
         playbooks += _build_playbook_list(
@@ -421,10 +424,41 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         self.run_kolla_ansible_seed(parsed_args, "bootstrap-servers",
                                     extra_vars=extra_vars)
 
+        # Re-run the Pip role after we've bootstrapped the Kolla user
+        extra_vars = {}
+        kolla_ansible_user = hostvars.get("kolla_ansible_user")
+        extra_vars["pip_applicable_users"] = [kolla_ansible_user]
+
         # Run final kayobe playbooks.
         playbooks = _build_playbook_list(
-            "kolla-target-venv", "kolla-host", "docker")
-        self.run_kayobe_playbooks(parsed_args, playbooks, limit="seed")
+            "pip", "kolla-target-venv", "kolla-host", "docker")
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  extra_vars=extra_vars, limit="seed")
+
+
+class SeedHostPackageUpdate(KayobeAnsibleMixin, VaultMixin, Command):
+    """Update packages on the seed host."""
+
+    def get_parser(self, prog_name):
+        parser = super(SeedHostPackageUpdate, self).get_parser(prog_name)
+        group = parser.add_argument_group("Host Package Updates")
+        group.add_argument("--packages", required=True,
+                           help="List of packages to update. Use '*' to "
+                                "update all packages.")
+        group.add_argument("--security", action='store_true',
+                           help="Only install updates that have been marked "
+                                "security related.")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Updating seed host packages")
+        extra_vars = {
+            "host_package_update_packages": parsed_args.packages,
+            "host_package_update_security": parsed_args.security,
+        }
+        playbooks = _build_playbook_list("host-package-update")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="seed",
+                                  extra_vars=extra_vars)
 
 
 class SeedHostUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
@@ -451,6 +485,7 @@ class SeedServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     * Deploys the bifrost container using kolla-ansible.
     * Builds disk images for the overcloud hosts using Diskimage Builder (DIB).
     * Performs a workaround in the overcloud host image to fix resolv.conf.
+    * Performs a workaround in the overcloud host image to update cloud-init
     * Configures ironic inspector introspection rules in the bifrost inspector
       service.
     * When enabled, configures a Bare Metal Provisioning (BMP) environment for
@@ -467,6 +502,7 @@ class SeedServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         self.run_kolla_ansible_seed(parsed_args, "deploy-bifrost")
         playbooks = _build_playbook_list(
             "overcloud-host-image-workaround-resolv",
+            "overcloud-host-image-workaround-cloud-init",
             "seed-introspection-rules",
             "dell-switch-bmp")
         self.run_kayobe_playbooks(parsed_args, playbooks)
@@ -501,6 +537,7 @@ class SeedServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         self.run_kolla_ansible_seed(parsed_args, "deploy-bifrost")
         playbooks = _build_playbook_list(
             "overcloud-host-image-workaround-resolv",
+            "overcloud-host-image-workaround-cloud-init",
             "seed-introspection-rules",
             "dell-switch-bmp")
         self.run_kayobe_playbooks(parsed_args, playbooks)
@@ -548,10 +585,22 @@ class SeedDeploymentImageBuild(KayobeAnsibleMixin, VaultMixin, Command):
     (DIB) for use when provisioning and inspecting the overcloud hosts.
     """
 
+    def get_parser(self, prog_name):
+        parser = super(SeedDeploymentImageBuild, self).get_parser(
+            prog_name)
+        group = parser.add_argument_group("Deployment Image Build")
+        group.add_argument("--force-rebuild", action="store_true",
+                           help="whether to force rebuilding the images")
+        return parser
+
     def take_action(self, parsed_args):
         self.app.LOG.debug("Building seed deployment images")
         playbooks = _build_playbook_list("seed-ipa-build")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
+        extra_vars = {}
+        if parsed_args.force_rebuild:
+            extra_vars["ipa_image_force_rebuild"] = True
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  extra_vars=extra_vars)
 
 
 class OvercloudInventoryDiscover(KayobeAnsibleMixin, VaultMixin, Command):
@@ -586,7 +635,7 @@ class OvercloudIntrospectionDataSave(KayobeAnsibleMixin, VaultMixin, Command):
     """Save hardware introspection data for the overcloud.
 
     Save hardware introspection data from the seed's ironic inspector service
-    to the control host.
+    to the Ansible control host.
     """
 
     def get_parser(self, prog_name):
@@ -676,6 +725,7 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     * Optionally, create a virtualenv for remote target hosts.
     * Optionally, wipe unmounted disk partitions (--wipe-disks).
     * Configure user accounts, group associations, and authorised SSH keys.
+    * Configure a PyPI mirror.
     * Configure Yum repos.
     * Disable SELinux.
     * Configure the host's network interfaces.
@@ -720,7 +770,7 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         # Kayobe playbooks.
         playbooks = _build_playbook_list(
             "ip-allocation", "ssh-known-host", "kayobe-ansible-user",
-            "kayobe-target-venv")
+            "pip", "kayobe-target-venv")
         if parsed_args.wipe_disks:
             playbooks += _build_playbook_list("wipe-disks")
         playbooks += _build_playbook_list(
@@ -748,10 +798,42 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         self.run_kolla_ansible_overcloud(parsed_args, "bootstrap-servers",
                                          extra_vars=extra_vars)
 
+        # Re-run the Pip role after we've bootstrapped the Kolla user
+        extra_vars = {}
+        kolla_ansible_user = hostvars.get("kolla_ansible_user")
+        extra_vars["pip_applicable_users"] = [kolla_ansible_user]
+
         # Further kayobe playbooks.
         playbooks = _build_playbook_list(
-            "kolla-target-venv", "kolla-host", "docker", "ceph-block-devices")
-        self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
+            "pip", "kolla-target-venv", "kolla-host",
+            "docker", "ceph-block-devices")
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  extra_vars=extra_vars, limit="overcloud")
+
+
+class OvercloudHostPackageUpdate(KayobeAnsibleMixin, VaultMixin, Command):
+    """Update packages on the overcloud hosts."""
+
+    def get_parser(self, prog_name):
+        parser = super(OvercloudHostPackageUpdate, self).get_parser(prog_name)
+        group = parser.add_argument_group("Host Package Updates")
+        group.add_argument("--packages", required=True,
+                           help="List of packages to update. Use '*' to "
+                                "update all packages.")
+        group.add_argument("--security", action='store_true',
+                           help="Only install updates that have been marked "
+                                "security related.")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Updating overcloud host packages")
+        extra_vars = {
+            "host_package_update_packages": parsed_args.packages,
+            "host_package_update_security": parsed_args.security,
+        }
+        playbooks = _build_playbook_list("host-package-update")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud",
+                                  extra_vars=extra_vars)
 
 
 class OvercloudHostUpgrade(KayobeAnsibleMixin, VaultMixin, Command):
@@ -891,7 +973,7 @@ class OvercloudServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
 
         # Deploy kayobe extra services.
         playbooks = _build_playbook_list("overcloud-extras")
-        extra_vars = {"action": "deploy"}
+        extra_vars = {"kayobe_action": "deploy"}
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   extra_vars=extra_vars)
 
@@ -948,7 +1030,7 @@ class OvercloudServiceReconfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
 
         # Reconfigure kayobe extra services.
         playbooks = _build_playbook_list("overcloud-extras")
-        extra_vars = {"action": "reconfigure"}
+        extra_vars = {"kayobe_action": "reconfigure"}
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   extra_vars=extra_vars)
 
@@ -1001,7 +1083,7 @@ class OvercloudServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin,
 
         # Upgrade kayobe extra services.
         playbooks = _build_playbook_list("overcloud-extras")
-        extra_vars = {"action": "upgrade"}
+        extra_vars = {"kayobe_action": "upgrade"}
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   extra_vars=extra_vars)
 
@@ -1047,7 +1129,7 @@ class OvercloudServiceDestroy(KollaAnsibleMixin, KayobeAnsibleMixin,
 
         # Destroy kayobe extra services.
         playbooks = _build_playbook_list("overcloud-extras")
-        extra_vars = {"action": "destroy"}
+        extra_vars = {"kayobe_action": "destroy"}
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   extra_vars=extra_vars)
 
@@ -1068,7 +1150,7 @@ class OvercloudContainerImagePull(KayobeAnsibleMixin, KollaAnsibleMixin,
 
         # Pull container images for kayobe extra services.
         playbooks = _build_playbook_list("overcloud-extras")
-        extra_vars = {"action": "pull"}
+        extra_vars = {"kayobe_action": "pull"}
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   extra_vars=extra_vars)
 
@@ -1107,10 +1189,22 @@ class OvercloudContainerImageBuild(KayobeAnsibleMixin, VaultMixin, Command):
 class OvercloudDeploymentImageBuild(KayobeAnsibleMixin, VaultMixin, Command):
     """Build the overcloud deployment kernel and ramdisk images."""
 
+    def get_parser(self, prog_name):
+        parser = super(OvercloudDeploymentImageBuild, self).get_parser(
+            prog_name)
+        group = parser.add_argument_group("Deployment Image Build")
+        group.add_argument("--force-rebuild", action="store_true",
+                           help="whether to force rebuilding the images")
+        return parser
+
     def take_action(self, parsed_args):
         self.app.LOG.debug("Building overcloud deployment images")
         playbooks = _build_playbook_list("overcloud-ipa-build")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
+        extra_vars = {}
+        if parsed_args.force_rebuild:
+            extra_vars["ipa_image_force_rebuild"] = True
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  extra_vars=extra_vars)
 
 
 class OvercloudPostConfigure(KayobeAnsibleMixin, VaultMixin, Command):
@@ -1173,3 +1267,40 @@ class BaremetalComputeProvide(KayobeAnsibleMixin, VaultMixin, Command):
         self.app.LOG.debug("Making baremetal compute nodes available")
         playbooks = _build_playbook_list("baremetal-compute-provide")
         self.run_kayobe_playbooks(parsed_args, playbooks)
+
+
+class BaremetalComputeRename(KayobeAnsibleMixin, VaultMixin, Command):
+    """Rename baremetal compute nodes to match inventory hostname"""
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Renaming baremetal compute nodes")
+        playbooks = _build_playbook_list("baremetal-compute-rename")
+        self.run_kayobe_playbooks(parsed_args, playbooks)
+
+
+class BaremetalComputeUpdateDeploymentImage(KayobeAnsibleMixin, VaultMixin,
+                                            Command):
+    """Update the Ironic nodes to use the new  kernel and ramdisk images."""
+
+    def get_parser(self, prog_name):
+        parser = super(BaremetalComputeUpdateDeploymentImage, self).get_parser(
+            prog_name)
+        group = parser.add_argument_group("Baremetal Compute Update")
+        group.add_argument("--baremetal-compute-limit",
+                           help="Limit the upgrade to the hosts specified in "
+                                "this limit"
+                           )
+        return parser
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug(
+            "Upgrading the ironic nodes to use the latest  deployment images")
+        playbooks = _build_playbook_list("overcloud-ipa-images")
+        extra_vars = {}
+        extra_vars["ipa_images_update_ironic_nodes"] = True
+        if parsed_args.baremetal_compute_limit:
+            extra_vars["ipa_images_compute_node_limit"] = (
+                parsed_args.baremetal_compute_limit
+            )
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  extra_vars=extra_vars)
